@@ -3,11 +3,11 @@ package jd
 func diff(
 	a, b JsonNode,
 	p Path,
-	options []Option,
+	opts *options,
 	strategy patchStrategy,
 ) Diff {
 	d := make(Diff, 0)
-	if a.Equals(b) {
+	if a.equals(b, opts) {
 		return d
 	}
 	var de DiffElement
